/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2014 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
*                                                                             *
******************************************************************************/

#include <includes.h>
#include <string.h>
#include <stdio.h>
#include <mcapi.h>

#define WAIT_TIMEOUT 0xFFFFFFFF

const int tx_port = 1000;
const int rx_port = 1001;

mcapi_pktchan_recv_hndl_t	send_handle;
mcapi_pktchan_recv_hndl_t	recv_handle;


/*
 * Macro to check on function return status
 */
#define mcapi_assert_success(s) \
	if (s != MCAPI_SUCCESS) { printf("%s:%d status %d\n", __FILE__, __LINE__, s); return; }

static void connect(int local, int remote)
{
	mcapi_endpoint_t local_send_endpoint;
	mcapi_endpoint_t local_recv_endpoint;
	mcapi_endpoint_t remote_recv_endpoint;
	mcapi_request_t  request;
	mcapi_request_t  send_request;
	mcapi_request_t  recv_request;
	mcapi_status_t   status;
	size_t           size;

	printf("Node %d: Creating tx port %d\n", local, tx_port);
	local_send_endpoint = mcapi_create_endpoint(tx_port, &status);
	mcapi_assert_success(status);

	printf("Node %d: Creating rx port %d\n", local, rx_port);
	local_recv_endpoint = mcapi_create_endpoint(rx_port, &status);
	mcapi_assert_success(status);

	printf("Node %d: Get remote rx port %d from node %d\n", local, rx_port, remote);
	remote_recv_endpoint = mcapi_get_endpoint(remote, rx_port, &status);
	mcapi_assert_success(status);

	printf("Node %d: Connecting %d:%d to %d:%d\n",local, local, tx_port,
	       remote, rx_port);
	mcapi_connect_pktchan_i(local_send_endpoint, remote_recv_endpoint,
	                        &request, &status);
	mcapi_assert_success(status);

	printf("Node %d: Waiting for connection setup with Node %d\n", local, rx_port);
	mcapi_wait(&request, &size, &status, WAIT_TIMEOUT);
	mcapi_assert_success(status);

	printf("Node %d: Connection complete\n", local);

	printf("Node %d: Opening send endpoint\n", local);
	mcapi_open_pktchan_send_i(&send_handle, local_send_endpoint, &send_request,
							  &status);

	printf("Node %d: Opening receive endpoint\n", local);
	mcapi_open_pktchan_recv_i(&recv_handle, local_recv_endpoint, &recv_request,
							  &status);

	mcapi_wait(&send_request, &size, &status, WAIT_TIMEOUT);
	mcapi_assert_success(status);

	mcapi_wait(&recv_request, &size, &status, WAIT_TIMEOUT);
	mcapi_assert_success(status);

	printf("Node %d: MCAPI negotiation complete! \n", local);
}

void disconnect(int local, int remote)
{
	mcapi_request_t  send_request;
	mcapi_request_t  recv_request;
	mcapi_status_t   status;
	size_t           size;

	printf("Node %d: Disconnecting ...\n", local);

	printf("Node %d: Closing send endpoint\n", local);
	mcapi_packetchan_send_close_i(send_handle, &send_request, &status);

	printf("Node %d: Closing receive endpoint\n", local);
	mcapi_packetchan_recv_close_i(recv_handle, &recv_request, &status);

	mcapi_wait(&send_request, &size, &status, WAIT_TIMEOUT);
	mcapi_assert_success(status);

	mcapi_wait(&recv_request, &size, &status, WAIT_TIMEOUT);
	mcapi_assert_success(status);

	printf("Node %d: MCAPI disconnection complete! \n", local);
}

void startup(unsigned int local, unsigned int remote)
{
	mcapi_status_t status;
	mcapi_version_t version;

	printf("Node %d: MCAPI Initializing\n",local);
	mcapi_initialize(local, &version, &status);
	mcapi_assert_success(status);
	printf("Node %d: MCAPI Initialized\n",local);

	connect(local, remote);
}

void cleanup(void)
{
	mcapi_status_t status;

	printf("%s\n", __func__);
	mcapi_finalize(&status);
}

void demo(unsigned int node, int loop)
{
	char outgoing[100];
	char *incoming;
	size_t bytes;
	mcapi_status_t status;
	int i = 0;

	do {
		memset(outgoing, 0, 100);
		sprintf(outgoing, "Hi from node %d - %d", node, i);

		mcapi_pktchan_send(send_handle, outgoing, strlen(outgoing)+1,
			&status);
		mcapi_assert_success(status);

		mcapi_pktchan_recv(recv_handle, (void *)&incoming, &bytes,
			&status);

		printf("Received msg %d: %s\n", i, incoming);
		i++;

		mcapi_pktchan_free(incoming, &status);
		mcapi_assert_success(status);

		OSTimeDlyHMSM(0, 0, 1, 0);
	} while (loop);
}
